package com.cultural.heritage.controller.good;


import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.model.dto.CommonBatchRequest;
import com.cultural.heritage.model.dto.cartService.CartExperienceAddRequest;
import com.cultural.heritage.model.dto.cartService.CartExperienceUpdateRequest;
import com.cultural.heritage.model.dto.order.BuySingleServiceGoodVerifyRequest;
import com.cultural.heritage.model.entity.CartExperience;
import com.cultural.heritage.model.entity.Good;
import com.cultural.heritage.model.entity.User;
import com.cultural.heritage.model.vo.cartService.CartExperienceGoodVO;
import com.cultural.heritage.model.vo.cartService.CartExperienceOrderVO;
import com.cultural.heritage.model.vo.cartService.CartExperienceVO;
import com.cultural.heritage.service.good.CartExperienceService;
import com.cultural.heritage.service.good.GoodService;
import com.cultural.heritage.service.user.UserService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.math.BigDecimal;
import java.util.*;

@RestController
@RequestMapping("/cartExperience")
@Slf4j
@Tag(name = "服务类购物车模块")
public class CartExperienceController {


    @Resource
    private CartExperienceService cartExperienceService;


    @Resource
    private GoodService goodService;


    @Resource
    private UserService userService;



    /**
     * 小程序端用户添加服务类商品至购物车
     * @param cartExperienceAddRequest  服务类购物车商品添加请求体
     * @return  是否添加成功
     */
    @PostMapping("/add")
    @Operation(summary = "小程序端用户添加服务类商品至购物车", description = "参数：服务类购物车商品添加请求体，权限：所有人，方法名：addCartRecord")
    public BaseResponse<Boolean> addCartRecord(@RequestBody CartExperienceAddRequest cartExperienceAddRequest, HttpServletRequest request) {
        if (cartExperienceAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 更新购物车的商品属性
        cartExperienceService.updateCartGoodAttribute(cartExperienceAddRequest, request);
        return ResultUtils.success(true);
    }


    /**
     * 小程序端用户更新服务类购物车中的商品数量
     * @param cartExperienceUpdateRequestList  服务类购物车更新请求体
     * @return  是否更新成功
     */
    @PostMapping("/update")
    @Operation(summary = "小程序端用户更新服务类购物车中的商品数量", description = "参数：服务类购物车更新请求体，权限：所有人，方法名：updateCartRecord")
    public BaseResponse<Boolean> updateCartRecord(@RequestBody List<CartExperienceUpdateRequest> cartExperienceUpdateRequestList, HttpServletRequest request) {
        if (CollectionUtils.isEmpty(cartExperienceUpdateRequestList)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        userService.getLoginUser(request);

        // 获取购物车id列表
        List<Long> cartIds = cartExperienceUpdateRequestList.stream().map(CartExperienceUpdateRequest::getId).toList();
        QueryWrapper<CartExperience> queryWrapper = new QueryWrapper<>();
        queryWrapper.in("id", cartIds);
        List<CartExperience> cartExperienceList = cartExperienceService.list(queryWrapper);

        // 封装map集合（键：购物车id, 值：商品数量）
        Map<Long, Integer> QuantityMap = new HashMap<>();
        // 封装map集合（键：购物车id, 值：预约时间）
        Map<Long, String> reservationDateMap = new HashMap<>();
        // 封装map集合（键：购物车id, 值：预约时间段）
        Map<Long, String> timeSlotMap = new HashMap<>();
        for (CartExperienceUpdateRequest cartExperienceUpdateRequest : cartExperienceUpdateRequestList) {
            Long id = cartExperienceUpdateRequest.getId();
            Integer quantity = cartExperienceUpdateRequest.getQuantity();
            String reservationDate = cartExperienceUpdateRequest.getReservationDate();
            String timeSlot = cartExperienceUpdateRequest.getTimeSlot();
            QuantityMap.put(id, quantity);
            reservationDateMap.put(id, reservationDate);
            timeSlotMap.put(id, timeSlot);
        }

        // 获取商品id列表
        List<Long> goodIds = cartExperienceList.stream().map(CartExperience::getGoodId).toList();
        QueryWrapper<Good> goodQueryWrapper = new QueryWrapper<>();
        goodQueryWrapper.in("id", goodIds);
        List<Good> goodList = goodService.list(goodQueryWrapper);

        // 封装map集合（键：商品id, 值：商品价格）
        Map<Long, BigDecimal> priceMap = new HashMap<>();
        for (Good good : goodList) {
            priceMap.put(good.getId(), good.getPrice());
        }

        // 批量更新购物车数量、小计、预约时间、预约时间段
        for (CartExperience cartExperience : cartExperienceList) {
            Long cartRecordId = cartExperience.getId();
            Integer quantity = QuantityMap.get(cartRecordId);
            cartExperience.setQuantity(quantity);

            Long goodId = cartExperience.getGoodId();
            BigDecimal price = priceMap.get(goodId);
            cartExperience.setSubtotal(price.multiply(BigDecimal.valueOf(quantity)));

            String reservationDate = reservationDateMap.get(cartRecordId);
            cartExperience.setReservationDate(reservationDate);
            String timeSlot = timeSlotMap.get(cartRecordId);
            cartExperience.setTimeSlot(timeSlot);
        }

        boolean result = cartExperienceService.updateBatchById(cartExperienceList);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "购物车信息更新失败");
        return ResultUtils.success(true);
    }



    /**
     * 小程序端用户删除服务类购物车中的物品
     * @param commonBatchRequest  删除的商品id列表
     * @return 是否删除成功
     */
    @PostMapping("/delete")
    @Operation(summary = "小程序端用户删除服务类购物车中的物品", description = "参数：购物车记录删除请求体(商品id列表，如[2,3,4])，权限：所有人，方法名：deleteCart")
    public BaseResponse<Boolean> deleteCart(@RequestBody CommonBatchRequest commonBatchRequest, HttpServletRequest request) {
        if (commonBatchRequest == null || commonBatchRequest.getIdList().isEmpty()) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        userService.getLoginUser(request);
        List<Long> idList = commonBatchRequest.getIdList();
        boolean result = cartExperienceService.removeBatchByIds(idList);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }



    /**
     * 小程序端用户查询服务类购物车中的商品
     * @return 商品列表信息
     */
    @PostMapping("/list")
    @Operation(summary = "小程序端用户查询服务类购物车中的商品", description = "参数：无，权限：所有人，方法名：listUserCartRecord")
    public BaseResponse<List<CartExperienceVO>> listUserCartRecord(HttpServletRequest request) {
        // 根据用户id查询购物车记录
        User loginUser = userService.getLoginUser(request);
        Long userId = loginUser.getId();
        QueryWrapper<CartExperience> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("userId", userId);
        queryWrapper.orderByDesc("id");
        List<CartExperience> cartExperienceList = cartExperienceService.list(queryWrapper);
        // 封装用户购物车中的商品信息
        List<CartExperienceVO> cartExperienceVOList = cartExperienceService.transformToCartRecordVOList(cartExperienceList);
        return ResultUtils.success(cartExperienceVOList);
    }




    /**
     * 小程序端显示服务类购物车中不合理的商品项
     * @return  存在问题的购物车id列表
     */
    @PostMapping("/submit/error")
    @Operation(summary = "小程序端显示服务类购物车中不合理的商品项", description = "参数:无，权限：所有人，方法名：listInvalidCartIds")
    public BaseResponse<List<Long>> listInvalidCartIds (HttpServletRequest request) {
        User loginUser = userService.getLoginUser(request);
        Long userId = loginUser.getId();
        QueryWrapper<CartExperience> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("userId", userId);
        List<CartExperience> cartExperienceList = cartExperienceService.list(queryWrapper);
        // 找出存在问题的购物车id列表
        List<Long> invalidCartIds = cartExperienceService.getInvalidCartIds(cartExperienceList);
        return ResultUtils.success(invalidCartIds);
    }





    /**
     * 小程序端显示服务类购物车提交订单页的商品信息
     * @param commonBatchRequest    购物车id列表
     * @return  订单商品信息列表
     */
    @PostMapping("/submit/list")
    @Operation(summary = "小程序端显示服务类购物车提交订单页的商品信息", description = "参数:商品id和购买数量信息，权限：所有人，方法名：listCartRecord")
    public BaseResponse<List<CartExperienceOrderVO>> listCartRecord(@RequestBody CommonBatchRequest commonBatchRequest, HttpServletRequest request) {
        if (commonBatchRequest == null || commonBatchRequest.getIdList().isEmpty()) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        userService.getLoginUser(request);
        List<Long> cartExperienceIds = commonBatchRequest.getIdList();
        List<CartExperience> cartExperienceList = cartExperienceService.list();
        // 获取购物车列表信息
        List<CartExperience> cartExperiences = cartExperienceList.stream().filter(cartExperience -> {
            Long id = cartExperience.getId();
            return cartExperienceIds.contains(id);
        }).toList();

        // 提取出购物车中的商品id列表
        List<Long> goodIds = cartExperiences.stream().map(CartExperience::getGoodId).toList();
        QueryWrapper<Good> queryWrapper = new QueryWrapper<>();
        queryWrapper.in("id", goodIds);
        List<Good> goodList = goodService.list(queryWrapper);

        // 封装goodMap(根据商品id查询商品)
        Map<Long, Good> goodMap = new HashMap<>();
        for (Good good : goodList) {
            goodMap.put(good.getId(), good);
        }

        // 封装提交订单页的商品信息列表
        List<CartExperienceOrderVO> cartOrderVOList = new ArrayList<>();
        for (CartExperience cartExperience : cartExperiences) {
            Integer quantity = cartExperience.getQuantity();
            Long goodId = cartExperience.getGoodId();
            Good good = goodMap.get(goodId);

            CartExperienceGoodVO cartExperienceGoodVO = new CartExperienceGoodVO();
            BeanUtils.copyProperties(good, cartExperienceGoodVO);

            CartExperienceOrderVO cartExperienceOrderVO = new CartExperienceOrderVO();
            cartExperienceOrderVO.setCartExperienceGoodVO(cartExperienceGoodVO);
            cartExperienceOrderVO.setQuantity(quantity);
            cartExperienceOrderVO.setReservationDate(cartExperience.getReservationDate());
            cartExperienceOrderVO.setTimeSlot(cartExperience.getTimeSlot());
            cartOrderVOList.add(cartExperienceOrderVO);
        }
        Collections.reverse(cartOrderVOList);
        return ResultUtils.success(cartOrderVOList);
    }





    /**
     * 小程序端用户点击立即购买触发的校验
     * @param buySingleServiceGoodVerifyRequest    商品id, 购买数量, 预约日期, 预约时间段
     * @return  是否通过校验
     */
    @PostMapping("/verify/good")
    @Operation(summary = "小程序端用户点击立即购买触发的校验", description = "参数:商品id, 购买数量, 预约日期, 预约时间段，权限：所有人，方法名：verifyPurchaseInfo")
    public BaseResponse<Boolean> verifyPurchaseInfo(@RequestBody BuySingleServiceGoodVerifyRequest buySingleServiceGoodVerifyRequest, HttpServletRequest request) {
        if (buySingleServiceGoodVerifyRequest == null || buySingleServiceGoodVerifyRequest.getGoodId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        userService.getLoginUser(request);
        Long goodId = buySingleServiceGoodVerifyRequest.getGoodId();
        Good good = goodService.getById(goodId);
        ThrowUtils.throwIf(good == null || good.getIsShelves() == 0, ErrorCode.SYSTEM_ERROR, "商品已下架或不存在");

        String reservationDate = buySingleServiceGoodVerifyRequest.getReservationDate();
        String timeSlot = buySingleServiceGoodVerifyRequest.getTimeSlot();

        String startTime = timeSlot.split("-")[0];
        String advanceDateTimeStr = reservationDate + " " + startTime + ":00";
        String currentDateTimeStr = DateUtil.now();

        Date advanceDateTime = DateUtil.parse(advanceDateTimeStr, "yyyy-MM-dd HH:mm:ss");
        Date currentDateTime = DateUtil.parse(currentDateTimeStr, "yyyy-MM-dd HH:mm:ss");

        int result = DateUtil.compare(advanceDateTime, currentDateTime);
        ThrowUtils.throwIf(result <= 0, ErrorCode.OPERATION_ERROR, "当前时间段已过期");

        return ResultUtils.success(true);
    }

}
