package com.cultural.heritage.service.good.impl;

import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.mapper.CartExperienceMapper;
import com.cultural.heritage.model.dto.cartService.CartExperienceAddRequest;
import com.cultural.heritage.model.dto.cartService.CartExperienceOrderItemAddRequest;
import com.cultural.heritage.model.entity.CartExperience;
import com.cultural.heritage.model.entity.Good;
import com.cultural.heritage.model.entity.User;
import com.cultural.heritage.model.vo.cartService.CartExperienceGoodVO;
import com.cultural.heritage.model.vo.cartService.CartExperienceVO;
import com.cultural.heritage.service.good.CartExperienceService;
import com.cultural.heritage.service.good.GoodService;
import com.cultural.heritage.service.user.UserService;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
@Slf4j
public class CartExperienceServiceImpl extends ServiceImpl<CartExperienceMapper, CartExperience> implements CartExperienceService {




    @Resource
    private GoodService goodService;


    @Resource
    private UserService userService;


    /**
     * 更新购物车的商品属性
     */
    @Override
    public boolean updateCartGoodAttribute(CartExperienceAddRequest cartExperienceAddRequest, HttpServletRequest request) {
        User loginUser = userService.getLoginUser(request);
        Long userId = loginUser.getId();
        Long goodId = cartExperienceAddRequest.getGoodId();
        Good good = goodService.getById(goodId);
        ThrowUtils.throwIf(good == null || good.getIsShelves() == 0, ErrorCode.OPERATION_ERROR, "商品不存在或者已下架");
        QueryWrapper<CartExperience> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("userId", userId);
        queryWrapper.eq("goodId", goodId);
        CartExperience cartExperience = this.baseMapper.selectOne(queryWrapper);
        if (cartExperience != null) {
            // 判断数量是否超出库存
            int currentQuantity = cartExperience.getQuantity() + cartExperienceAddRequest.getQuantity();
            if (currentQuantity > good.getInventory()) {
                throw new BusinessException(ErrorCode.SYSTEM_ERROR, "商品库存不足, 无法添加");
            }
            // 如果购物车中已存在该商品, 就叠加数量
            cartExperience.setQuantity(currentQuantity);
            cartExperience.setSubtotal(good.getPrice().multiply(BigDecimal.valueOf(currentQuantity)));
            boolean result = this.updateById(cartExperience);
            ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "更新购物车商品数量失败");
        } else {
            // 判断数量是否超出库存
            Integer currentQuantity = cartExperienceAddRequest.getQuantity();
            if (currentQuantity > good.getInventory()) {
                throw new BusinessException(ErrorCode.SYSTEM_ERROR, "商品库存不足, 无法添加");
            }
            // 如果购物车不存在该商品，就向购物车中添加一项
            CartExperience cartExperienceRecord = new CartExperience();
            BeanUtils.copyProperties(cartExperienceAddRequest, cartExperienceRecord);
            cartExperienceRecord.setUserId(userId);
            cartExperienceRecord.setSubtotal(good.getPrice().multiply(BigDecimal.valueOf(cartExperienceRecord.getQuantity())));
            // 校验
            this.validCart(cartExperienceRecord, false);
            boolean result = this.save(cartExperienceRecord);
            ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        }
        return true;
    }





    /**
     * 封装用户购物车中的商品信息
     */
    @Override
    public List<CartExperienceVO> transformToCartRecordVOList(List<CartExperience> cartExperienceList) {

        // 获取所有商品id
        List<Long> goodIds = cartExperienceList.stream().map(CartExperience::getGoodId).toList();
        QueryWrapper<Good> queryWrapper = new QueryWrapper<>();
        queryWrapper.in("id", goodIds);
        List<Good> goodList = goodService.list(queryWrapper);
        // 封装map集合（键：商品id, 值：商品详情信息）
        Map<Long, Good> map = new HashMap<>();
        for (Good good : goodList) {
            map.put(good.getId(), good);
        }

        return cartExperienceList.stream().map(cartExperience -> {
            CartExperienceVO cartExperienceVO = new CartExperienceVO();
            BeanUtils.copyProperties(cartExperience, cartExperienceVO);

            CartExperienceGoodVO cartExperienceGoodVO = new CartExperienceGoodVO();
            Long goodId = cartExperience.getGoodId();
            Good good = map.get(goodId);
            BeanUtils.copyProperties(good, cartExperienceGoodVO);

            cartExperienceVO.setCartExperienceGoodVO(cartExperienceGoodVO);
            return cartExperienceVO;
        }).toList();
    }



    /**
     * 校验购物车中的商品信息
     */
    @Override
    public void validCart(CartExperience cartExperience, boolean update) {
        Long userId = cartExperience.getUserId();
        Long goodId = cartExperience.getGoodId();
        Integer quantity = cartExperience.getQuantity();
        BigDecimal subtotal = cartExperience.getSubtotal();
        String reservationDate = cartExperience.getReservationDate();
        String timeSlot = cartExperience.getTimeSlot();
        if (update) {
            if (userId == null) {
                throw new BusinessException(ErrorCode.PARAMS_ERROR);
            }
        }

        Good good = goodService.getById(goodId);
        if (good == null || good.getIsShelves() == 0) {
            throw new BusinessException(ErrorCode.NOT_FOUND_ERROR, "不存在该商品或该商品已下架");
        }
        if (StringUtils.isAnyBlank(reservationDate, timeSlot)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "预约日期或者预约时间段参数为空");
        }

        String startTime = timeSlot.split("-")[0];
        String advanceDateTimeStr = reservationDate + " " + startTime + ":00";
        String currentDateTimeStr = DateUtil.now();

        Date advanceDateTime = DateUtil.parse(advanceDateTimeStr, "yyyy-MM-dd HH:mm:ss");
        Date currentDateTime = DateUtil.parse(currentDateTimeStr, "yyyy-MM-dd HH:mm:ss");

        int result = DateUtil.compare(advanceDateTime, currentDateTime);
        ThrowUtils.throwIf(result <= 0, ErrorCode.OPERATION_ERROR, "当前时间段已过期");

        if (ObjectUtils.isEmpty(quantity) || quantity < 1) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "请传递正确的商品数量");
        }
        if (ObjectUtils.isEmpty(subtotal) || subtotal.compareTo(BigDecimal.ZERO) <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "价格错误");
        }
    }


    /**
     * 校验购物车商品项的准确性
     */
    private boolean validIsConsistent(CartExperience cartExperience, Good good) {
        // 校验购物车中的商品是否存在
        if (good == null || good.getIsShelves() == 0) {
            log.info("商品不存在或者已下架, 购物车id为" + cartExperience.getId());
            return false;
        }
        // 判断当前时间是否晚于预约时间
        String reservationDate = cartExperience.getReservationDate();
        String startTime = cartExperience.getTimeSlot().split("-")[0];
        String advanceDateTimeStr = reservationDate + " " + startTime + ":00";
        String currentDateTimeStr = DateUtil.now();

        Date advanceDateTime = DateUtil.parse(advanceDateTimeStr, "yyyy-MM-dd HH:mm:ss");
        Date currentDateTime = DateUtil.parse(currentDateTimeStr, "yyyy-MM-dd HH:mm:ss");

        int result = DateUtil.compare(advanceDateTime, currentDateTime);
        return result > 0;
    }


    /**
     * 在提交订单页校验购物车商品项的准确性
     */
    @Override
    public boolean validIsConsistent(List<CartExperienceOrderItemAddRequest> cartExperienceOrderItemAddRequestList) {
        if (cartExperienceOrderItemAddRequestList.size() == 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "购物车中没有商品");
        }
        // 封装map集合（键：购物车id, 值：购买数量）
        Map<Long, Integer> map = new HashMap<>();
        for (CartExperienceOrderItemAddRequest cartExperienceOrderItemAddRequest : cartExperienceOrderItemAddRequestList) {
            Long cartExperienceId = cartExperienceOrderItemAddRequest.getCartExperienceId();
            Integer quantity = cartExperienceOrderItemAddRequest.getQuantity();
            map.put(cartExperienceId, quantity);
        }
        // 修改购物车商品项的购买数量
        List<Long> cartExperienceIds = cartExperienceOrderItemAddRequestList.stream().map(CartExperienceOrderItemAddRequest::getCartExperienceId).toList();
        QueryWrapper<CartExperience> queryWrapper = new QueryWrapper<>();
        queryWrapper.in("id", cartExperienceIds);
        List<CartExperience> cartExperiences = this.list(queryWrapper);
        for (CartExperience cartExperience : cartExperiences) {
            Long id = cartExperience.getId();
            Integer quantity = map.get(id);
            cartExperience.setQuantity(quantity);
        }
        List<Long> cartIds = getInvalidCartIds(cartExperiences);
        return cartIds.isEmpty();
    }


    /**
     * 找出存在问题的购物车id列表
     */
    @Override
    public List<Long> getInvalidCartIds(List<CartExperience> cartExperienceList) {
        // 获取购物车的商品id列表
        List<Long> goodIds = cartExperienceList.stream().map(CartExperience::getGoodId).toList();
        QueryWrapper<Good> queryWrapper = new QueryWrapper<>();
        queryWrapper.in("id", goodIds);
        List<Good> goodList = goodService.list(queryWrapper);
        // 封装map集合（键：商品id, 值：商品详情信息）
        Map<Long, Good> map = new HashMap<>();
        for (Good good : goodList) {
            map.put(good.getId(), good);
        }
        List<CartExperience> cartExperiences = cartExperienceList.stream().filter(cartExperience -> {
            Long goodId = cartExperience.getGoodId();
            Good good = map.get(goodId);
            return !validIsConsistent(cartExperience, good);
        }).toList();
        return cartExperiences.stream().map(CartExperience::getId).toList();
    }

}
